//
//	Author				: CYTRON	
//	Project				: PR23 + SKPS
//	Project description	: mobile robot controlled using PS (PR23+SKPS)

//	include
//=======================================================================
#include <pic.h> 

//	configuration
//=======================================================================
__CONFIG ( 0x3F32 );

//	define
//=======================================================================

#define motor_ra	RC0
#define motor_rb	RC3
#define motor_la	RC4
#define motor_lb	RC5

#define buzzer 		RE2
#define SPEEDL		CCPR1L
#define SPEEDR		CCPR2L

#define	rs			RB7
#define	e			RB6
#define b_light		RB5
#define	lcd_data	PORTD

#define buzzer 		RE2
#define	CHANNEL0	0b10000001	// AN0
#define	CHANNEL1	0b10001001	// AN1

//skps protocol
#define	p_select		0
#define p_joyl			1
#define p_joyr			2
#define p_start			3
#define p_up			4
#define p_right			5
#define p_down			6
#define p_left			7
#define	p_l2			8
#define	p_r2			9
#define p_l1			10
#define p_r1			11
#define p_triangle		12
#define p_circle		13
#define p_cross			14
#define	p_square		15
#define p_joy_lx		16
#define	p_joy_ly		17
#define p_joy_rx		18
#define p_joy_ry		19
#define p_joy_lu		20
#define p_joy_ld		21
#define p_joy_ll		22
#define p_joy_lr		23
#define p_joy_ru		24
#define p_joy_rd		25
#define p_joy_rl		26
#define p_joy_rr		27

#define	p_con_status	28
#define p_motor1		29
#define p_motor2		30

//	global variable
//=======================================================================



//	function prototype
//=======================================================================
void init(void);
void delay(unsigned long data);
void send_config(unsigned char data);
void send_char(unsigned char data);
void e_pulse(void);
void lcd_goto(unsigned char data);
void lcd_clr(void);
void send_string(const char *s);
unsigned char read_adc(char config);
void forward(void);
void stop (void);
void backward (void);
void reverse (void);
void left(void);
void right(void);

void uart_send(unsigned char data);
unsigned char uart_rec(void);
unsigned char skps(unsigned char data);
void skps_vibrate(unsigned char motor, unsigned char value);

//	interrupt prototype
//=======================================================================
static void interrupt isr(void)
{

} 

//	main function
//=======================================================================
void main(void)
{
	unsigned char up_v, down_v, left_v, right_v;
	
	init();			//initialize microcontroller
	stop();			//stop all motor
	
	lcd_clr();
	lcd_goto(0);
	send_string("Controlling PR23");
	lcd_goto(20);
	send_string("   Using SKPS");
	
	while(1)
	{	
		//test button for horn
		if(skps(p_l1)==0)buzzer=1;			//if button L1 pressed, beep the buzzer
		else buzzer=0;						
		
		//test if there is any obstacle in front
		if(read_adc(CHANNEL0)<20)			//if ultrasonic sensors detect obstacle in front,
		{									//vibrate the ps controller
			skps_vibrate(p_motor2,255);	
		}
		else skps_vibrate(p_motor2,0);

		//read joy stick value process		
		up_v=skps(p_joy_ru);
		down_v=skps(p_joy_rd);
		left_v=skps(p_joy_ll);
		right_v=skps(p_joy_lr);		
				
		//button control for mobilility
		if(skps(p_up)==0)					//check "up" button
		{
			forward();						//move forward
			SPEEDL=230;
			SPEEDR=230;
		}
		else if(skps(p_down)==0)			//check "down" button
		{	
			backward();						//move backward
			SPEEDL=230;
			SPEEDR=230;
		}
		else if(skps(p_left)==0)			//check "left" button
		{
			left();							//rotate left
			SPEEDL=230;
			SPEEDR=230;
		}
		else if(skps(p_right)==0)			//check "right" button
		{
			right();						//rotate right
			SPEEDL=230;
			SPEEDR=230;
		}
				
		//analog control for mobility	
		else if(up_v>0)
		{
			forward();
			if(left_v>0)
			{
				if(up_v>left_v)SPEEDL=up_v-left_v+140;
				else SPEEDL=140;
				SPEEDR=up_v+140;		
			}
			else if(right_v>0)
			{
				if(up_v>right_v)SPEEDR=up_v-right_v+140;
				else SPEEDR=140;
				SPEEDL=up_v+140;
			}
			else
			{
				SPEEDL=up_v+140;
				SPEEDR=up_v+140;	
			}
		}
		else if(down_v>0)
		{
			backward();
			if(left_v>0)
			{
				if(down_v>left_v)SPEEDR=down_v-left_v+140;
				else SPEEDR=140;
				SPEEDL=down_v+140;		
			}
			else if(right_v>0)
			{
				if(down_v>right_v)SPEEDL=down_v-right_v+140;
				else SPEEDL=140;
				SPEEDR=down_v+140;
			}
			else
			{
				SPEEDL=down_v+140;
				SPEEDR=down_v+140;	
			}
		}
		else if(left_v>0)
		{
			left();
			SPEEDL=left_v+120;
			SPEEDR=left_v+120;	
		}
		else if(right_v>0)
		{
			right();
			SPEEDL=right_v+120;
			SPEEDR=right_v+120;	
		}
		else
		{
			stop();
			SPEEDL=255;
			SPEEDR=255;	
		}
	}
}

// initailization
//=======================================================================
void init()
{
	// ADC configuration	
	ADCON1 = 0b10000100;				//set RA0 and RA1 as Analog Input, left justified

	// setup for capture pwm
	RBIE = 1;							// enable interrupt on change of port B

	// motor PWM configuration 
	PR2 = 255;						// set period register 
	T2CON =  	0b00000100;			// 
	CCP1CON =	0b00001100;			// config for RC1 to generate PWM	( for more detail refer datasheet section 'capture/compare/pwm')
	CCP2CON =	0b00001100;			// config for RC2 to generate PWM

	// Tris configuration (input or output)
	TRISA = 0b00000011;				//set RA0 and RA2 pin as input,other as output
	TRISB = 0b00011111;				//set RB0-RB4 pin as input, other as output
	TRISC = 0b10000000;				//set PORTC pin as output
	TRISD = 0b00000000;				//set all PORTD pin as output
	TRISE = 0b00000011;

	// TMR 0 configuation
	T0CS = 0;						
	PSA = 0;						
	PS2 = 1;						// prescale 1:32
	PS1 = 1;						//
	PS0 = 1;						//
	TMR0IE = 0;						// TMR0 Interrupt
	TMR0 = 0;

	//setup USART
	SPBRG = 0x81;						//set baud rate to 9600 for 20Mhz
	BRGH = 1;							//baud rate high speed option
	TXEN = 1;							//enable transmission
	TX9 = 0;
	CREN = 1;							//enable reception
	SPEN = 1;							//enable serial port
	RX9 = 0;
	RCIE = 0;							//disable interrupt on eachdata received

	// enable all unmasked interrupt	
	GIE = 0;
	PEIE = 0;

	// LCD configuration
	send_config(0b00000001);		//clear display at lcd
	send_config(0b00000010);		//Lcd Return to home 
	send_config(0b00000110);		//entry mode-cursor increase 1
	send_config(0b00001100);		//diplay on, cursor off and cursor blink off
	send_config(0b00111000);		//function

	b_light = 0;
	buzzer = 0;
}

// uart function
//=======================================================================
void uart_send(unsigned char data)	//function to send out a byte via uart
{	
	while(TXIF==0);					//wait for previous data to finish send out
	TXREG=data;						//send new data
}

unsigned char uart_rec(void)		//function to wait for a byte receive from uart
{
	unsigned char temp;
	while(RCIF==0);					//wait for data to received
	temp=RCREG;					
	return temp;					//return the received data
}

// skps function
//=======================================================================
unsigned char skps(unsigned char data)			//function to read button and joystick 
{												//information on ps controller
	uart_send(data);
	return uart_rec();
}

void skps_vibrate(unsigned char motor, unsigned char value)
{												//function to control the vibrator motor 
	uart_send(motor);							//on ps controller
	uart_send(value);	
}

// read adc
//=======================================================================
unsigned char read_adc(char config)
{
	unsigned short i;
	unsigned long result_temp=0;
	unsigned char result;

	ADCON0 = config;
	delay(10000);					// delay after changing configuration 
	for(i=200;i>0;i-=1)				//looping 200 times for getting average value 
	{
		ADGO = 1;					//ADGO is the bit 2 of the ADCON0 register
		while(ADGO==1);				//ADC start, ADGO=0 after finish ADC progress
		result=ADRESH;
		result=result<<8;			//shift to left for 8 bit
		result=result|ADRESL;		//10 bit result from ADC
		result_temp+=result;		
	}
	result = result_temp/200;		//getting average value
	ADON = 0;						//adc module is shut off
	return result;
}

// motor control function
//=======================================================================
void forward ()
{
	motor_ra = 0;
	motor_rb = 1;
	motor_la = 0;
	motor_lb = 1;
}

void backward ()
{
	motor_ra = 1;
	motor_rb = 0;
	motor_la = 1;
	motor_lb = 0;
}

void left()
{
	motor_la = 1;
	motor_lb = 0;
	motor_ra = 0;
	motor_rb = 1;
}
void right()
{
	motor_la = 0;
	motor_lb = 1;
	motor_ra = 1;
	motor_rb = 0;
}

void stop()
{
	motor_la = 1;
	motor_lb = 1;
	motor_ra = 1;
	motor_rb = 1;
}


//	LCD	functions
//=======================================================================
void delay(unsigned long data)
{
	for( ;data>0;data-=1);
}

void send_config(unsigned char data)
{
	rs=0;										//clear rs into config mode 
	lcd_data=data;
	delay(300);
	e_pulse();
}

void send_char(unsigned char data)
{
	rs=1;										//set rs into write mode
	lcd_data=data;						 
	delay(300);
	e_pulse();
}

void e_pulse(void)
{
	e=1;
	delay(300);
	e=0;
	delay(300);
}

void lcd_goto(unsigned char data)
{
 	if(data<16)
	{
	 	send_config(0x80+data);
	}
	else
	{
	 	data=data-20;
		send_config(0xc0+data);
	}
}

void lcd_clr(void)
{
 	send_config(0x01);
	delay(350);	
}

void send_string(const char *s)
{          
	unsigned char i=0;
  	while (s && *s)send_char (*s++);

}


